@@ -9,7 +9,6 @@ import (
 	"fmt"
 	"os"
 	"strconv"
-	"strings"
 	"sync"
 	"sync/atomic"
 
@@ -45,7 +44,7 @@ type overlay struct {
 	uri     span.URI
 	text    []byte
 	hash    string
-	version float64
+	version int32
 	kind    source.FileKind
 
 	// saved is true if a file matches the state on disk,
@@ -81,7 +80,7 @@ func (o *overlay) URI() span.URI {
 	return o.uri
 }
 
-func (o *overlay) Version() float64 {
+func (o *overlay) Version() int32 {
 	return o.version
 }
 
@@ -114,7 +113,7 @@ func (c *closedFile) Session() string {
 	return ""
 }
 
-func (c *closedFile) Version() float64 {
+func (c *closedFile) Version() int32 {
 	return 0
 }
 
@@ -175,14 +174,14 @@ func (s *Session) createView(ctx context.Context, name string, folder, tempWorks
 	}
 	root := folder
 	if options.ExpandWorkspaceToModule {
-		root, err = findWorkspaceRoot(ctx, root, s, options.ExperimentalWorkspaceModule)
+		root, err = findWorkspaceRoot(ctx, root, s, pathExcludedByFilterFunc(options), options.ExperimentalWorkspaceModule)
 		if err != nil {
 			return nil, nil, func() {}, err
 		}
 	}
 
 	// Build the gopls workspace, collecting active modules in the view.
-	workspace, err := newWorkspace(ctx, root, s, options.ExperimentalWorkspaceModule)
+	workspace, err := newWorkspace(ctx, root, s, pathExcludedByFilterFunc(options), ws.userGo111Module == off, options.ExperimentalWorkspaceModule)
 	if err != nil {
 		return nil, nil, func() {}, err
 	}
@@ -199,12 +198,11 @@ func (s *Session) createView(ctx context.Context, name string, folder, tempWorks
 		id:                   strconv.FormatInt(index, 10),
 		options:              options,
 		baseCtx:              baseCtx,
-		backgroundCtx:        backgroundCtx,
-		cancel:               cancel,
 		name:                 name,
 		folder:               folder,
-		filesByURI:           make(map[span.URI]*fileBase),
-		filesByBase:          make(map[string][]*fileBase),
+		moduleUpgrades:       map[string]string{},
+		filesByURI:           map[span.URI]*fileBase{},
+		filesByBase:          map[string][]*fileBase{},
 		rootURI:              root,
 		workspaceInformation: *ws,
 		tempWorkspace:        tempWorkspace,
@@ -218,6 +216,8 @@ func (s *Session) createView(ctx context.Context, name string, folder, tempWorks
 	v.snapshot = &snapshot{
 		id:                snapshotID,
 		view:              v,
+		backgroundCtx:     backgroundCtx,
+		cancel:            cancel,
 		initializeOnce:    &sync.Once{},
 		generation:        s.cache.store.Generation(generationName(v, 0)),
 		packages:          make(map[packageKey]*packageHandle),
@@ -231,7 +231,6 @@ func (s *Session) createView(ctx context.Context, name string, folder, tempWorks
 		unloadableFiles:   make(map[span.URI]struct{}),
 		parseModHandles:   make(map[span.URI]*parseModHandle),
 		modTidyHandles:    make(map[span.URI]*modTidyHandle),
-		modUpgradeHandles: make(map[span.URI]*modUpgradeHandle),
 		modWhyHandles:     make(map[span.URI]*modWhyHandle),
 		workspace:         workspace,
 	}
@@ -288,12 +287,11 @@ func (s *Session) viewOf(uri span.URI) (*View, error) {
 		return v, nil
 	}
 	// Pick the best view for this file and memoize the result.
-	v, err := s.bestView(uri)
-	if err != nil {
-		return nil, err
+	if len(s.views) == 0 {
+		return nil, fmt.Errorf("no views in session")
 	}
-	s.viewMap[uri] = v
-	return v, nil
+	s.viewMap[uri] = bestViewForURI(uri, s.views)
+	return s.viewMap[uri], nil
 }
 
 func (s *Session) viewsOf(uri span.URI) []*View {
@@ -302,7 +300,7 @@ func (s *Session) viewsOf(uri span.URI) []*View {
 
 	var views []*View
 	for _, view := range s.views {
-		if strings.HasPrefix(string(uri), string(view.Folder())) {
+		if source.InDir(view.folder.Filename(), uri.Filename()) {
 			views = append(views, view)
 		}
 	}
@@ -319,15 +317,12 @@ func (s *Session) Views() []source.View {
 	return result
 }
 
-// bestView finds the best view to associate a given URI with.
-// viewMu must be held when calling this method.
-func (s *Session) bestView(uri span.URI) (*View, error) {
-	if len(s.views) == 0 {
-		return nil, errors.Errorf("no views in the session")
-	}
+// bestViewForURI returns the most closely matching view for the given URI
+// out of the given set of views.
+func bestViewForURI(uri span.URI, views []*View) *View {
 	// we need to find the best view for this file
 	var longest *View
-	for _, view := range s.views {
+	for _, view := range views {
 		if longest != nil && len(longest.Folder()) > len(view.Folder()) {
 			continue
 		}
@@ -336,16 +331,16 @@ func (s *Session) bestView(uri span.URI) (*View, error) {
 		}
 	}
 	if longest != nil {
-		return longest, nil
+		return longest
 	}
 	// Try our best to return a view that knows the file.
-	for _, view := range s.views {
+	for _, view := range views {
 		if view.knownFile(uri) {
-			return view, nil
+			return view
 		}
 	}
 	// TODO: are there any more heuristics we can use?
-	return s.views[0], nil
+	return views[0]
 }
 
 func (s *Session) removeView(ctx context.Context, view *View) error {
@@ -405,7 +400,7 @@ func (s *Session) dropView(ctx context.Context, v *View) (int, error) {
 }
 
 func (s *Session) ModifyFiles(ctx context.Context, changes []source.FileModification) error {
-	_, _, releases, _, err := s.DidModifyFiles(ctx, changes)
+	_, releases, err := s.DidModifyFiles(ctx, changes)
 	for _, release := range releases {
 		release()
 	}
@@ -418,21 +413,13 @@ type fileChange struct {
 	fileHandle source.VersionedFileHandle
 }
 
-func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModification) (map[span.URI]source.View, map[source.View]source.Snapshot, []func(), []span.URI, error) {
+func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModification) (map[source.Snapshot][]span.URI, []func(), error) {
 	views := make(map[*View]map[span.URI]*fileChange)
-	bestViews := map[span.URI]source.View{}
-	// Keep track of deleted files so that we can clear their diagnostics.
-	// A file might be re-created after deletion, so only mark files that
-	// have truly been deleted.
-	deletions := map[span.URI]struct{}{}
-
-	// If the set of changes included directories, expand those directories
-	// to their files.
-	changes = s.expandChangesToDirectories(ctx, changes)
+	affectedViews := map[span.URI][]*View{}
 
 	overlays, err := s.updateOverlays(ctx, changes)
 	if err != nil {
-		return nil, nil, nil, nil, err
+		return nil, nil, err
 	}
 	var forceReloadMetadata bool
 	for _, c := range changes {
@@ -441,12 +428,6 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 		}
 
 		// Build the list of affected views.
-		bestView, err := s.viewOf(c.URI)
-		if err != nil {
-			return nil, nil, nil, nil, err
-		}
-		bestViews[c.URI] = bestView
-
 		var changedViews []*View
 		for _, view := range s.views {
 			// Don't propagate changes that are outside of the view's scope
@@ -456,17 +437,24 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 			}
 			changedViews = append(changedViews, view)
 		}
-		// If no view matched the change, assign it to the best view.
+		// If the change is not relevant to any view, but the change is
+		// happening in the editor, assign it the most closely matching view.
 		if len(changedViews) == 0 {
+			if c.OnDisk {
+				continue
+			}
+			bestView, err := s.viewOf(c.URI)
+			if err != nil {
+				return nil, nil, err
+			}
 			changedViews = append(changedViews, bestView)
 		}
+		affectedViews[c.URI] = changedViews
 
 		// Apply the changes to all affected views.
 		for _, view := range changedViews {
 			// Make sure that the file is added to the view.
-			if _, err := view.getFile(c.URI); err != nil {
-				return nil, nil, nil, nil, err
-			}
+			_ = view.getFile(c.URI)
 			if _, ok := views[view]; !ok {
 				views[view] = make(map[span.URI]*fileChange)
 			}
@@ -476,11 +464,10 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 					exists:     true,
 					fileHandle: fh,
 				}
-				delete(deletions, c.URI)
 			} else {
 				fsFile, err := s.cache.getFile(ctx, c.URI)
 				if err != nil {
-					return nil, nil, nil, nil, err
+					return nil, nil, err
 				}
 				content, err := fsFile.Read()
 				fh := &closedFile{fsFile}
@@ -489,32 +476,39 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 					exists:     err == nil,
 					fileHandle: fh,
 				}
-				// If there was an error reading the file, assume it has been
-				// deleted.
-				if err != nil {
-					deletions[c.URI] = struct{}{}
-				}
 			}
 		}
 	}
 
-	snapshots := map[source.View]source.Snapshot{}
 	var releases []func()
+	viewToSnapshot := map[*View]*snapshot{}
 	for view, changed := range views {
 		snapshot, release := view.invalidateContent(ctx, changed, forceReloadMetadata)
-		snapshots[view] = snapshot
 		releases = append(releases, release)
+		viewToSnapshot[view] = snapshot
 	}
-	var deletionsSlice []span.URI
-	for uri := range deletions {
-		deletionsSlice = append(deletionsSlice, uri)
+
+	// We only want to diagnose each changed file once, in the view to which
+	// it "most" belongs. We do this by picking the best view for each URI,
+	// and then aggregating the set of snapshots and their URIs (to avoid
+	// diagnosing the same snapshot multiple times).
+	snapshotURIs := map[source.Snapshot][]span.URI{}
+	for _, mod := range changes {
+		viewSlice, ok := affectedViews[mod.URI]
+		if !ok || len(viewSlice) == 0 {
+			continue
+		}
+		view := bestViewForURI(mod.URI, viewSlice)
+		snapshot, ok := viewToSnapshot[view]
+		if !ok {
+			panic(fmt.Sprintf("no snapshot for view %s", view.Folder()))
+		}
+		snapshotURIs[snapshot] = append(snapshotURIs[snapshot], mod.URI)
 	}
-	return bestViews, snapshots, releases, deletionsSlice, nil
+	return snapshotURIs, releases, nil
 }
 
-// expandChangesToDirectories returns the set of changes with the directory
-// changes removed and expanded to include all of the files in the directory.
-func (s *Session) expandChangesToDirectories(ctx context.Context, changes []source.FileModification) []source.FileModification {
+func (s *Session) ExpandModificationsToDirectories(ctx context.Context, changes []source.FileModification) []source.FileModification {
 	var snapshots []*snapshot
 	for _, v := range s.views {
 		snapshot, release := v.getSnapshot(ctx)
@@ -626,7 +620,7 @@ func (s *Session) updateOverlays(ctx context.Context, changes []source.FileModif
 		}
 		// On-disk changes don't come with versions.
 		version := c.Version
-		if c.OnDisk {
+		if c.OnDisk || c.Action == source.Save {
 			version = o.version
 		}
 		hash := hashContents(text)
@@ -636,7 +630,7 @@ func (s *Session) updateOverlays(ctx context.Context, changes []source.FileModif
 			// Do nothing. sameContentOnDisk should be false.
 		case source.Save:
 			// Make sure the version and content (if present) is the same.
-			if o.version != version {
+			if false && o.version != version { // Client no longer sends the version
 				return nil, errors.Errorf("updateOverlays: saving %s at version %v, currently at %v", c.URI, c.Version, o.version)
 			}
 			if c.Text != nil && o.hash != hash {
@@ -702,3 +696,15 @@ func (s *Session) Overlays() []source.Overlay {
 	}
 	return overlays
 }
+
+func (s *Session) FileWatchingGlobPatterns(ctx context.Context) map[string]struct{} {
+	patterns := map[string]struct{}{}
+	for _, view := range s.views {
+		snapshot, release := view.getSnapshot(ctx)
+		for k, v := range snapshot.fileWatchingGlobPatterns(ctx) {
+			patterns[k] = v
+		}
+		release()
+	}
+	return patterns
+}
