@@ -521,7 +521,7 @@ func (r *runner) SuggestedFix(t *testing.T, spn span.Span, actionKinds []string,
 		// Hack: We assume that we only get one code action per range.
 		var cmds []string
 		for _, a := range actions {
-			cmds = append(cmds, fmt.Sprintf("%s (%s)", a.Command.Command, a.Title))
+			cmds = append(cmds, fmt.Sprintf("%s (%s)", a.Command, a.Title))
 		}
 		t.Fatalf("unexpected number of code actions, want %d, got %d: %v", expectedActions, len(actions), cmds)
 	}
